@@ -14,23 +14,11 @@
 #ifndef LLVM_CONFIG_H
 #define LLVM_CONFIG_H
 
-/*
- * When bsd.lib.mk builds shared libraries it builds with -DPIC which 
- * causes problems in the following files which use PIC as a variable name.
- * undefine PIC here to minimise the diff to upstream LLVM
- *
- * include/llvm/MC/MCObjectFileInfo.h
- * lib/MC/MCObjectFileInfo.cpp
- * lib/Transforms/Scalar/LICM.cpp
- * lib/Transforms/Utils/PredicateInfo.cpp
- */
-#undef PIC
-
 /* Define if LLVM_ENABLE_DUMP is enabled */
 /* #undef LLVM_ENABLE_DUMP */
 
 /* Target triple LLVM will generate code for by default */
-#define LLVM_DEFAULT_TARGET_TRIPLE "sparc64-unknown-openbsd6.8"
+#define LLVM_DEFAULT_TARGET_TRIPLE "powerpc64-unknown-openbsd7.0"
 
 /* Define if threads enabled */
 #define LLVM_ENABLE_THREADS 1
@@ -39,28 +27,28 @@
 #define LLVM_HAS_ATOMICS 1
 
 /* Host triple LLVM will be executed on */
-#define LLVM_HOST_TRIPLE "sparc64-unknown-openbsd6.8"
+#define LLVM_HOST_TRIPLE "powerpc64-unknown-openbsd7.0"
 
 /* LLVM architecture name for the native architecture, if available */
-#define LLVM_NATIVE_ARCH Sparc
+#define LLVM_NATIVE_ARCH PowerPC
 
 /* LLVM name for the native AsmParser init function, if available */
-#define LLVM_NATIVE_ASMPARSER LLVMInitializeSparcAsmParser
+#define LLVM_NATIVE_ASMPARSER LLVMInitializePowerPCAsmParser
 
 /* LLVM name for the native AsmPrinter init function, if available */
-#define LLVM_NATIVE_ASMPRINTER LLVMInitializeSparcAsmPrinter
+#define LLVM_NATIVE_ASMPRINTER LLVMInitializePowerPCAsmPrinter
 
 /* LLVM name for the native Disassembler init function, if available */
-#define LLVM_NATIVE_DISASSEMBLER LLVMInitializeSparcDisassembler
+#define LLVM_NATIVE_DISASSEMBLER LLVMInitializePowerPCDisassembler
 
 /* LLVM name for the native Target init function, if available */
-#define LLVM_NATIVE_TARGET LLVMInitializeSparcTarget
+#define LLVM_NATIVE_TARGET LLVMInitializePowerPCTarget
 
 /* LLVM name for the native TargetInfo init function, if available */
-#define LLVM_NATIVE_TARGETINFO LLVMInitializeSparcTargetInfo
+#define LLVM_NATIVE_TARGETINFO LLVMInitializePowerPCTargetInfo
 
 /* LLVM name for the native target MC init function, if available */
-#define LLVM_NATIVE_TARGETMC LLVMInitializeSparcTargetMC
+#define LLVM_NATIVE_TARGETMC LLVMInitializePowerPCTargetMC
 
 /* Define if this is Unixish platform */
 #define LLVM_ON_UNIX 1
@@ -75,20 +63,26 @@
 #define LLVM_USE_PERF 0
 
 /* Major version of the LLVM API */
-#define LLVM_VERSION_MAJOR 10
+#define LLVM_VERSION_MAJOR 11
 
 /* Minor version of the LLVM API */
-#define LLVM_VERSION_MINOR 0
+#define LLVM_VERSION_MINOR 1
 
 /* Patch version of the LLVM API */
-#define LLVM_VERSION_PATCH 1
+#define LLVM_VERSION_PATCH 0
 
 /* LLVM version string */
-#define LLVM_VERSION_STRING "10.0.1"
+#define LLVM_VERSION_STRING "11.1.0"
 
 /* Whether LLVM records statistics for use with GetStatistics(),
  * PrintStatistics() or PrintStatisticsJSON()
  */
 #define LLVM_FORCE_ENABLE_STATS 0
 
+/* Define if we have z3 and want to build it */
+/* #undef LLVM_WITH_Z3 */
+
+/* Define if LLVM was built with a dependency to the libtensorflow dynamic library */
+/* #undef LLVM_HAVE_TF_API */
+
 #endif
